@@ -1,14 +1,29 @@
 AgrMcDt <- function(MicDtDF,agrby,agrcrt="minmax")
 {
-  if (!(is.data.frame(MicDtDF))) stop("First argument of AgMicroData must be a data frame\n")
-#  if (class(agrby)!="factor") stop("Argument agrby is not a factor\n")
-  if (class(agrby)[1]!="factor") stop("Argument agrby is not a factor\n")
+  mcall <- match.call()$MicDtDF
+  if (length(mcall) > 1) mcall <- "microdata"
+  if (!(is.data.frame(MicDtDF))) stop("First argument of AgMicroData must be a data frame.\n")
+  if (class(MicDtDF)[1]!="data.frame") MicDtDF <- as.data.frame(MicDtDF)
+  if (any(!sapply(1:ncol(MicDtDF),function(ind) is.numeric(MicDtDF[,ind])))) {  
+    stop(paste("Some of the columns of the",mcall,"data frame have non-numeric variables.\n"))
+  }
+  
+  unvalidobs <- which(apply(MicDtDF,1,function(v) any(!is.finite(v))))
+  nunvalid <- length(unvalidobs) 
+  if (nunvalid>0) {
+    MicDtDF <- MicDtDF[-unvalidobs,]
+    agrby <- agrby[-unvalidobs]
+    string2 <- paste("rows of the",mcall,"data frame were dropped because they included non-valid (non finite or missing values) observations.\n")
+    if (nunvalid<=10) warning(paste("The",paste(row.names(MicDtDF)[unvalidobs],collapse=" "),string2))
+    else warning(paste(nunvalid,string2,collapse=" "))
+  }
+
+  if (!is.factor(agrby)) stop("Argument agrby is not a factor\n")
   globaln <- nrow(MicDtDF)
-  if (length(agrby)!=globaln) stop("Size of the agrby argument does not agree with the number of rows in the MicDtDF data frame\n") 
-#To  do: test if all columns of MicDtDF are numeric
-#  if ( agrcrt[1]!="minmax" && (class(agrcrt)!="numeric" || length(agrcrt)!=2 || agrcrt[1]>=agrcrt[2] || agrcrt[1]<0. || agrcrt[2]>1.) )
+  if (length(agrby)!=globaln) stop("Size of the agrby argument does not agree with the number of rows in the MicDtDF data frame.\n") 
   if ( agrcrt[1]!="minmax" && (class(agrcrt)[1]!="numeric" || length(agrcrt)!=2 || agrcrt[1]>=agrcrt[2] || agrcrt[1]<0. || agrcrt[2]>1.) )
-    stop("Wrong value for the agrcrt argument\n( it should be either the string minmax or a two-dim vector\nof a prob. value for the lower percentile, followed by the prob. value for the upper percentile - \nex:c(0.05,0.95) )\n") 
+    stop(paste("Wrong value for the agrcrt argument\n( it should be either the string minmax or a two-dim vector",
+               "\nof a prob. value for the lower percentile, followed by the prob. value for the upper percentile - \nex:c(0.05,0.95) ).\n")) 
    
   if (length(unique(agrby))!=length(levels(agrby)))  agrby <- factor(agrby)
   grplvls <- levels(agrby)
@@ -17,10 +32,12 @@ AgrMcDt <- function(MicDtDF,agrby,agrcrt="minmax")
 
   ngrps <- length(grplvls)
   nvar <- ncol(MicDtDF)
+  NbMicroUnits <- integer(ngrps)
   for (r in 1:ngrps) 
   { 
     grp <- grplvls[r]
     rind <- which(agrby==grp)
+    NbMicroUnits[r] <- length(rind)
     for (c in 1:nvar) {
       if (agrcrt[1]=="minmax") {
         bndsDF[r,c] <- min(MicDtDF[rind,c])
@@ -31,7 +48,7 @@ AgrMcDt <- function(MicDtDF,agrby,agrcrt="minmax")
       }
     }  
   }
-  res <- IData(bndsDF,Seq="AllLb_AllUb",VarNames=names(MicDtDF),ObsNames=grplvls) 
+  res <- IData(bndsDF,Seq="AllLb_AllUb",VarNames=names(MicDtDF),ObsNames=grplvls)
   DegInT <- which(apply(res@LogR,1,function(v) any(!is.finite(v))))
   nDegInT <- length(DegInT)
   if (nDegInT>0) {
@@ -51,8 +68,11 @@ AgrMcDt <- function(MicDtDF,agrby,agrcrt="minmax")
       wmsg <- paste(nDegInT,"data units were eliminated because they lead to some degenerate intervals")
     }
     warning(wmsg)
-    return(res[-DegInT,])
+    res <- res[-DegInT,]
   }
+  res@NbMicroUnits <- NbMicroUnits[-DegInT] 
+  names(res@NbMicroUnits) <- res@ObsNames
+  
   res
 }
 
