@@ -8,8 +8,7 @@ DACrossVal <- function(data,grouping,TrainAlg,EvalAlg=EvalClrule,Strfolds=TRUE,k
     lb:ub  # return(lb:ub)
   }   
 
-#  if (class(grouping)!="factor") {
-  if (class(grouping)[1]!="factor") {
+  if (!is.factor(grouping)) {
     grouping <- factor(grouping)
   }
   codes <- levels(grouping)
@@ -49,7 +48,6 @@ DACrossVal <- function(data,grouping,TrainAlg,EvalAlg=EvalClrule,Strfolds=TRUE,k
       }
       if (length(out)==0) next
       tres <- try(TrainAlg(data[-out,,drop=FALSE],grouping[-out],prior=prior,...))
-#      if (is.null(tres) || class(tres) == "try-error")
       if (is.null(tres) || class(tres)[1] == "try-error")
       {
         EvalResij <- list(err=NA,Nk=as.numeric(table(grouping[-out])))
